// Gray - A simple ray tracing program
// Copyright (C) 2008-2018 Eric Bechet
//
// See the COPYING file for contributions and license information.
// Please report all bugs and problems to <bechet@cadxfem.org>.
//

#include "glframebuffer.h"
#include "nglu.h"

void gl_framebuffer::display(bool perspectiveon)
{
  Fl_Window *win =new Fl_Window ( getwidth(), getheight(),"Gray - the homemade raytracer" );
  nglwindow * gl = new nglwindow ( 0,0,win->w(), win->h(),ncolor(0,0,0,0), this );
  setogl(gl);


  if ( scene()->getCam() == NULL )
  {
    std::cout << "No camera has been configured. Interrupting." << std::endl;
    return ;
  }

  Gscene *sc=scene();
  Camera *cam = sc->getCam();

  double lens = cam->getLens();
  double focal = cam->getFocal();

  double fov = atan ( lens/2/focal ) * 2 * 180 / 3.14159265;
  Point viewpoint = sc->getViewpoint(); // eye position

  GLdouble vp[3];
  vp[0]=viewpoint.X;
  vp[1]=viewpoint.Y;
  vp[2]=viewpoint.Z;

  GLdouble look[3];
  look[0] = cam->getDirection().X;
  look[1] = cam->getDirection().Y;
  look[2] = cam->getDirection().Z;

  GLdouble up[3];
  up[0] = cam->getUp().X;
  up[1] = cam->getUp().Y;
  up[2] = cam->getUp().Z;
  gl->set_view(lens,focal,fov,cam->getNearclip(),cam->getFarclip(),vp,look,up);
//  glPixelStorei ( GL_UNPACK_ALIGNMENT, 1 );
  win->end();
  win->resizable ( gl );
  win->show();  // this actually opens the window
  Fl::run();
  delete gl;
  delete win;
}



void gl_framebuffer::get_coords ( float i, float j, Ray &r )
{
  npoint3 p,v;
  double z;
  ogl->get_coords ( i,j,0.0f,p);
  ogl->get_coords ( i,j,1.0f,v);
  r.p.X=p.x();r.p.Y=p.y();r.p.Z=p.z();
  r.v.X=p.x()-v.x();r.v.Y=p.y()-v.y();r.v.Z=p.z()-v.z();
  r.v.normalize();
}

void gl_framebuffer::draw_model(int mode)
{
  glDisable(GL_COLOR_MATERIAL);
  int i=0;
  if ( scene() ) for ( Gscene::iteratorLight it= scene()->beginLight(); it!=scene()->endLight(); ++it,++i )
  {
    if ( i>=8 ) break;
    ( *it )->draw_opengl ( i );
  }
  
  if ( scene() ) for ( Gscene::iteratorModel it= scene()->beginModel(); it!=scene()->endModel(); ++it ) ( *it )->draw_opengl();
  if (!empty) draw_pixmap();
  empty=true;
}

void gl_framebuffer::draw_pixmap()
{
  if ( ok() )
  {
    glMatrixMode ( GL_PROJECTION );
    glPushMatrix();
    glLoadIdentity();
    glOrtho ( 0,getwidth(),0,getheight(),0,1 );
    glMatrixMode ( GL_MODELVIEW );
    glPushMatrix();
    glLoadIdentity();
    glRasterPos2i ( 0, 0 );
    glDrawPixels ( getwidth(), getheight(), GL_RGB, GL_FLOAT, getbuffer() );
    glPopMatrix();
    glMatrixMode ( GL_PROJECTION );
    glPopMatrix();
  }
}

int gl_framebuffer::handle(int event)
{
  Gscene *sc=scene();

  switch ( event )
  {
    case FL_KEYBOARD:
    {
      switch ( Fl::event_key() )
      {
        case 114 :// r
        {
          resize ( getwidth(),getheight() );
          ogl->get_matrices();
          makeimage();
          /*      std::ofstream file("output.bin",std::ios::binary);
                fb->save_image(file);
                file.close();*/
          ogl->redraw();
          return 1;
        }
        default : 
          ogl->redraw();
      }
      return 0;
    }

    default:
      return 0;
  }
}
// kate: indent-mode cstyle; indent-width 2; replace-tabs on; 
