// Gray - A simple ray tracing program
// Copyright (C) 2008-2018 Eric Bechet
//
// See the COPYING file for contributions and license information.
// Please report all bugs and problems to <bechet@cadxfem.org>.
//

#ifdef WIN32
#include "windows.h"
#endif //WIN32
#include "glu_gray.h"
#include <GL/gl.h>
#include <cmath>
#include <cstring>
#include "FL/math.h"

glmeshbuffer::glmeshbuffer ( int nv,int nt, int nq )
{
  vertices.reserve ( nv );
  normals.reserve ( nv );
  triangles.reserve ( nt );
  quads.reserve ( nq );
}

int glmeshbuffer::add_vertex ( Point v,Point n )
{
  vertices.push_back ( v );
  normals.push_back ( n );
  return vertices.size()-1;
}

int glmeshbuffer::add_triangle ( int a,int b,int c )
{
  triangles.push_back ( Tri ( a,b,c ) );
  return triangles.size()-1;
}

int glmeshbuffer::add_quad ( int a,int b,int c, int d )
{
  quads.push_back ( Quad ( a, b, c, d ) );
  return quads.size() - 1;
}

void glmeshbuffer::draw ( void )
{
  glEnableClientState ( GL_NORMAL_ARRAY );
  glEnableClientState ( GL_VERTEX_ARRAY );
  glNormalPointer ( GL_DOUBLE, 0, &normals[0] );
  glVertexPointer ( 3, GL_DOUBLE, 0, &vertices[0] );

  // Triangles-support
  glDrawElements ( GL_TRIANGLES, triangles.size() *3, GL_UNSIGNED_INT, &triangles[0] );

  // Quads-support
  glDrawElements ( GL_QUADS, quads.size() * 4, GL_UNSIGNED_INT, &quads[0] );

  glDisableClientState ( GL_VERTEX_ARRAY ); // disable vertex arrays
  glDisableClientState ( GL_NORMAL_ARRAY );
}


// kate: indent-mode cstyle; indent-width 2; replace-tabs on; 
