// Gnurbs - A curve and surface library
// Copyright (C) 2008-2018 Eric Bechet
//
// See the LICENSE file for contributions and license information.
// Please report all bugs and problems to <bechet@cadxfem.org>.
//

#ifndef __NSPLINE_H
#define __NSPLINE_H


#include "nlagrange.h"
#include <vector>

/// Spline class.
class nspline : public nlagrange
{
protected:
  std::vector<npoint> der;
  double card; //!< Parameter in [0, 1] for the cardinal spline type.
  enum {TypeUnknown,FiniteDifferences,Cardinal,Natural} type; 
    //!< type of spline (defines the way derivatives are computed )
public:

  /// \brief Constructor.
  /// \param[in] nCP_ number of points to interpolate.
  /// \warning Spline and interpolation points are unitialized.
  nspline(int nCP_) : nlagrange(nCP_),der(nCP_),type(TypeUnknown),card(0) {}

  /// \brief Computes the finite-difference spline.
  void compute_FD(void);

  /// \brief Computes the cardinal spline.
  /// \param[in] c cardinal parameter in [0, 1].
  void compute_cardinal(double c);

  /// \brief Computes the natural spline.
  void compute_natural(void);
  virtual int degree(void) const
  {
    return 3 ;
  }
  virtual void P(double u_,npoint& ret) const ;    // position along the curve
  virtual void set_CP(int which,const npoint& pt)  // sets a control point
  {
    val[which]=pt;
    update();
  }

  /// \brief Recomputes derivatives from point positions.
  virtual void update();
  virtual nspline* clone() const
  {
    return new nspline(*this);
  }
private :
  void tridiagsolve(std::vector<double>& rhs);
};


#endif // __NSPLINE_H
