// nUtil - An utility Library for gnurbs
// Copyright (C) 2008-2018 Eric Bechet
//
// See the LICENSE file for contributions and license information.
// Please report all bugs and problems to <bechet@cadxfem.org>.
//

#include "nglwindow.h"
#include "nglu.h"
#include "GL/glu.h"
#include <algorithm>
#include <limits>


int nglwindow::handle ( int event ) // called from fltk
{
//  std::cout << event << std::endl ;
  if (!caller->handle(event))// if event not handled by caller, 
  switch ( event )
  {
    case FL_MOVE:
    {
//      std::cout << "move" << std::endl ;
      return 1;
    }
      
    case FL_PUSH:
//    ... mouse down event ...
//    ... position in Fl::event_x() and Fl::event_y()
    {
      npoint3 p;
      npoint3 v;
      GLdouble xpp,ypp,zz;
      xp=Fl::event_x();
      yp=h()-Fl::event_y();
      get_coords ( xp,yp, p,v,z);
      pickedpoint=p;
//      std::cout << "z1=" << z << std::endl;
      if (caller->allow_pick)
      {
        int r;
        std::vector<pickinfo> picks;
        int nbpicks=pick(xp,yp,picks);
//        std::cout << nbpicks << std::endl;
        if (nbpicks>0)
        {
          int whichpick=0;
          if (caller->ncb)
          {
            whichpick=caller->ncb->pick_callback(nbpicks,&picks[0]);
          }
            
          if (whichpick>=0)
          {
            picking=true;
            current_pick=picks[whichpick];
            // correct z value given by get_coords which does not work for transparent objects)
//            z=((float)current_pick.z)/std::numeric_limits< unsigned int >::max();
//            std::cout << "z2=" << z << std::endl << std::endl;
            get_coords_with_z( xp,yp,z, p,v);
            pickedpoint=p;
          }
        }
      }

      orig=p;
      dinit=0.;
      for (int i=0;i<3;++i) {dinit+=(look[i]-vp[i])*(look[i]-vp[i]);}
      dinit=sqrt(dinit);
      orig=p;
//      redraw();
      return 1;
    }

    case FL_DRAG:  //    ... mouse moved while down event ...
    {
      if (Fl::event_button1())
      {
        npoint3 p;
        npoint3 dir;
        int xx= Fl::event_x();
        int yy=h()-Fl::event_y();
        get_coords(xx,yy,z, p);
        if (picking&&caller->allow_pick)
        {
          if (caller->ncb)
          {
            int res=caller->ncb->drag_callback(pickedpoint,p,current_pick);
            if (res) redraw();
          }
        }
        else
        {
          double d=0.0;
          for (int i=0;i<3;++i) {d+=(look[i]-vp[i]+(p[i]-orig[i])*5)*(look[i]-vp[i]+(p[i]-orig[i])*5);}
          d=sqrt(d);
          for (int i=0;i<3;++i) { vp[i]-=(p[i]-orig[i])*5;orig[i]=p[i];}
          for (int i=0;i<3;++i) { vp[i]=(vp[i]-look[i])*dinit/d + look[i];}
          d=0.0;
          for (int i=0;i<3;++i) { dir[i]=vp[i]-look[i] ; d+=dir[i]*dir[i];}
          d=sqrt(d);
          for (int i=0;i<3;++i) { dir[i]/=d;}
          double pscal=0;
          for (int i=0;i<3;++i) { pscal+=dir[i]*up[i];}
          d=0.0;
          for (int i=0;i<3;++i) { up[i]-=dir[i]*pscal;d+=up[i]*up[i];}
          d=sqrt(d);
          for (int i=0;i<3;++i) { up[i]/=d;}
          set_view();
          redraw();
        }
        return 1;
      }
      else if (Fl::event_button2())
      {
        npoint3 p;
        int xx= Fl::event_x();
        int yy=h()-Fl::event_y();
        get_coords (xx,yy,z, p);
        if (picking)
        {
          
        }
        else
        {
          for (int i=0;i<3;++i) { look[i]-=p[i]-orig[i];vp[i]-=p[i]-orig[i];orig[i]=p[i];}
          set_view();
          redraw();
        }
        return 1;
      }
    }

    case FL_RELEASE: //    ... mouse up event ...
    { 
      npoint3 p;
      npoint3 dir;
      int xx= Fl::event_x();
      int yy=h()-Fl::event_y();
      get_coords(xx,yy,z, p);
      if (picking&&caller->allow_pick)
      {
        if (caller->ncb)
        {
          int res=caller->ncb->release_callback(pickedpoint,p,current_pick);
          if (res) redraw();
        }
      }
      get_matrices();
      picking=false;
      return 1;
    }
    case FL_FOCUS :
    case FL_UNFOCUS : //    ... Return 1 if you want keyboard events, 0 otherwise
      return 1;

    case FL_MOUSEWHEEL:
    {
      if (Fl::event_shift())
      {
        npoint3 p;
        npoint3 v;
        xp=Fl::event_x();
        yp=h()-Fl::event_y();
        get_coords ( xp,yp, p,v,z);
        orig=p;
        double d=0.0;
        for (int i=0;i<3;++i) {d+=(look[i]-vp[i])*(look[i]-vp[i]);}
        d=sqrt(d);
        d/=10.;
        for (int i=0;i<3;++i) {look[i]-=v[i]*Fl::event_dy()*d;vp[i]-=v[i]*Fl::event_dy()*d;}
      }
        else
      {
        focal*=(1.0-Fl::event_dy()/10.0);
        fov=(( lens/2/focal ) * 2 * 180 / 3.14159265);
      }
    
      set_view();
      redraw();
      get_matrices();
      return 1;
    }
    case FL_KEYBOARD:
    {
      int key=Fl::event_key();
      const char*txt=Fl::event_text();
      switch ( key )
      {
        case FL_Shift_L :
        case FL_Shift_R :     // event_shift
        {
          caller->allow_pick=!caller->allow_pick;
          break;
        }
        default : {  
          int ret1=0,ret2=0;
          if (caller->ncb)
          {
             ret1= caller->ncb->key_callback(key);
             ret2= caller->ncb->key_callback(txt);
          }
          if (!ret1&&!ret2) std::cout << "unpicked key #" << key <<  " " << txt << std::endl;
          else redraw();
          return 1;

        }
      }

//    ... keypress, key is in Fl::event_key(), ascii in Fl::event_text()
//    ... Return 1 if you understand/use the keyboard event, 0 otherwise...
      return 1;
    }

    case FL_SHORTCUT:
//    ... shortcut, key is in Fl::event_key(), ascii in Fl::event_text()
//    ... Return 1 if you understand/use the shortcut event, 0 otherwise...
      return 1;

    default:
      // pass other events to the base class...
      return Fl_Gl_Window::handle ( event );
  }
  else return 1;
}


void nglwindow::draw_model ( int mode ) // called automatically at every refresh
{
  caller->draw_model(mode);
}


int nglwindow::pick (int x, int y,std::vector<pickinfo> &picks)
{
  int nb_obj=caller->getnumpick();
//  std::cout << " max pick=" << caller->getnumpick() << std::endl;
  int size=nb_obj*8+10;
  GLuint *sel = new GLuint[size];
  glSelectBuffer(size, sel);
  glRenderMode(GL_SELECT);
  glMatrixMode(GL_PROJECTION);
  glPushMatrix();
  glLoadIdentity();
  set_view(DRAW_PICK,x,y,5,5);
  draw_model(DRAW_PICK);
  glMatrixMode(GL_PROJECTION);
  glPopMatrix();
  GLint numhits = glRenderMode(GL_RENDER);

// Here, GL names have two to five levels:
// the 0 to 3 first are user ids, then the two following are local ids 
// first is the type of the entity (1 for point, 2 for edge, 3 for triangle 4 for quad
// and the second is the entity number in the data storage.
  
  unsigned int hi = 0;
  GLuint *bufp = sel;
  GLuint name, numnames, zmin, zmax;
  if (numhits<0) // overflow : too many picks
  {
    delete[] sel;
    return -1;
  }
  for(unsigned int j = 0; j < numhits; j++)
  {
    numnames = *bufp++;
    zmin = *bufp++;
    zmax = *bufp++;
    GLuint z=zmax;
//    std::cout << "z1z2" << std::endl;
//    std::cout << (float)z1/std::numeric_limits< unsigned int >::max() << " " << (float)z2/std::numeric_limits< unsigned int >::max() << std::endl;
//    std::cout << ((float)z1+z2)/(2.0*std::numeric_limits< unsigned int >::max()) << std::endl;
    if (numnames>=2&&numnames<=5)
    {
      pickinfo p;
      p.nb_uid=numnames-2;
      p.z=z;
      for (int i=0;i<p.nb_uid;++i)
        p.uids[i]=*bufp++;
      p.type=*bufp++;
      p.id=*bufp++;
      picks.push_back(p);
    }
    else
    {
      while(numnames--)  // skip go to next record
      {
        bufp++;
      }
    }
  }
  delete[] sel;
  std::sort(picks.begin(),picks.end()); // closest to eye in front.
/*  std::cout << "num=" << picks.size() << std::endl;
  for(unsigned int j = 0; j < picks.size(); j++)
    std::cout << picks[j].type << " " << picks[j].id << " " << picks[j].z  << std::endl;*/
  return picks.size();
}


void  nglwindow::draw ( void )  // called from fltk
{
  if ( !valid() )
  {
    set_view();
    get_matrices();
    print_info();
  }

  #ifndef MESA
  glDrawBuffer(GL_FRONT_AND_BACK);
  #endif // !MESA
  glClear ( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
  draw_model();
  #ifndef MESA
  glDrawBuffer(GL_BACK);
  #endif // !MESA
}

bool nglwindow::get_coords ( GLfloat i,GLfloat j, npoint3 &p,npoint3 &v,GLfloat &z)
{
  npoint3 p2;
  bool object=true;
  z = 0.0f;
  glReadBuffer(GL_FRONT);
  glReadPixels(i, j, 1, 1, GL_DEPTH_COMPONENT, GL_FLOAT, &z);
  if (z==1.0f) // no object or out of frustrum : we take the point at which we look to as reference distance
  {
    object=false;
    double finalMatrix[16];
    double xpp,ypp,zz;
    _gluMultMatricesd ( ( GLdouble * )ModelViewMatrix, ( GLdouble * )ProjectionMatrix,( GLdouble * ) finalMatrix );
    _gluProjectFast(look[0],look[1],look[2],finalMatrix,( GLint * )Viewport,&xpp,&ypp,&zz);
    z=zz;
  }
  _gluUnProjectFast ( i*1.0,j*1.0,z, ( GLdouble* ) Inv, ( GLint * ) Viewport,&p[0], &p[1], &p[2] );
  _gluUnProjectFast ( i*1.0,j*1.0,0.0, ( GLdouble* ) Inv, ( GLint * ) Viewport,&p2[0], &p2[1], &p2[2]);
  v=p-p2;
  v.normalize();
  if (z==1.0f) // no object or out of frustrum
  {
    p=p;
  }
  return object; // object found or not
}

bool nglwindow::get_coords_with_z ( GLfloat i,GLfloat j, GLfloat z,npoint3 &p,npoint3 &v)
{
  npoint3 p2;
  bool object=true;
  if (z==1.0f) // no object or out of frustrum : we take the point at which we look to as reference distance
  {
    object=false;
    double finalMatrix[16];
    double xpp,ypp,zz;
    _gluMultMatricesd ( ( GLdouble * )ModelViewMatrix, ( GLdouble * )ProjectionMatrix,( GLdouble * ) finalMatrix );
    _gluProjectFast(look[0],look[1],look[2],finalMatrix,( GLint * )Viewport,&xpp,&ypp,&zz);
    z=zz;
  }
  _gluUnProjectFast ( i*1.0,j*1.0,z, ( GLdouble* ) Inv, ( GLint * ) Viewport,&p[0], &p[1], &p[2] );
  _gluUnProjectFast ( i*1.0,j*1.0,0.0, ( GLdouble* ) Inv, ( GLint * ) Viewport,&p2[0], &p2[1], &p2[2]);
  v=p-p2;
  v.normalize();
  if (z==1.0f) // no object or out of frustrum
  {
    p=p;
  }
  return object; // object found or not
}

bool nglwindow::get_coords ( GLfloat di,GLfloat dj,GLfloat z_, npoint3 &p)
{
  npoint3 p2;
  _gluUnProjectFast ( di*1.0,dj*1.0,z_, ( GLdouble* ) Inv, ( GLint * ) Viewport,&p2[0], &p2[1], &p2[2]);
  p=p2;
  return true;
}


void nglwindow::get_matrices ( void )
{
  glGetDoublev ( GL_MODELVIEW_MATRIX, ( GLdouble * ) ModelViewMatrix );
  glGetDoublev ( GL_PROJECTION_MATRIX, ( GLdouble * ) ProjectionMatrix );
  glGetIntegerv ( GL_VIEWPORT, ( GLint * ) Viewport );
  _gluMultMatricesd ( ( GLdouble * ) ModelViewMatrix, ( GLdouble * )  ProjectionMatrix, ( GLdouble * ) Inv );
  _gluInvertMatrixd ( ( GLdouble * ) Inv, ( GLdouble * ) Inv );
}


void nglwindow::set_view (double lens_,double focal_, double fov_,double nearclip_,double farclip_,const double vp_[3],const double look_[3],const double up_[3])
{
  lens=lens_;focal=focal_,fov=fov_,nearclip=nearclip_,farclip=farclip_;
  for (int i=0;i<3;++i) {vp[i]=vp_[i];look[i]=look_[i];up[i]=up_[i];}
  set_view();
}

void nglwindow::set_view (int mode,int i,int j,int di,int dj)
{
//  glDepthFunc ( GL_LESS );
  glEnable ( GL_DEPTH_TEST );
  glEnable( GL_BLEND ); 
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA); 
  glPolygonMode ( GL_FRONT_AND_BACK,GL_FILL);
  glLoadIdentity();
  glViewport ( 0,0,w(),h() );
  glOrtho ( 0,w(),0,h(),0,1 );
  // couleur de fond (RGBT)
  glClearColor ( c.R/255., c.G/255., c.B/255.,c.A/255.);
  glClearAccum(0.0,0.0,0.0,0.0);
  // Remplissage
//  glShadeModel ( GL_FLAT);
  glShadeModel ( GL_SMOOTH );

  glMatrixMode ( GL_PROJECTION );
//  glPushMatrix();
  glLoadIdentity();
  
  if (mode==DRAW_PICK)
  {
    GLint viewport[4];
    glGetIntegerv(GL_VIEWPORT,viewport);
    gluPickMatrix(i, j,
                   di,dj, viewport);
//    std::cout << "pickij= " << i << " " <<  j << " " << std::endl;
  }
  _gluPerspective ( fov, ( GLdouble ) w() / ( GLdouble ) h(),
                      nearclip, farclip );
  glMatrixMode ( GL_MODELVIEW );
//  glPushMatrix();
  glLoadIdentity();
  _gluLookAt ( vp,
                  look,    /* lookat point */
                  up );  /* up is in +ive y */


  glEnable(GL_LINE_SMOOTH);
  glEnable(GL_POINT_SMOOTH);
  glEnable ( GL_LIGHTING );
  glEnable(GL_COLOR_MATERIAL);
  glEnable(GL_LIGHTING);
  add_lights();  
}

void nglwindow::add_lights ( void )
{
  glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
  GLfloat light_ambient[] =
  {0.0, 0.0, 0.0, 0.0};
  GLfloat light_diffuse[] =
  {1.0, 1.0, 1.0, 1.0};
  GLfloat light_specular[] =
  {0.0, 0.0, 0.0, 1.0};
  GLfloat light_position[] =
  {(GLfloat)(vp[0]-look[0]),(GLfloat)(vp[1]-look[1]),(GLfloat)(vp[2]-look[2]), 0.0};
  glLightfv ( GL_LIGHT0, GL_AMBIENT, light_ambient );
  glLightfv ( GL_LIGHT0, GL_DIFFUSE, light_diffuse );
  glLightfv ( GL_LIGHT0, GL_SPECULAR, light_specular );
  glLightfv ( GL_LIGHT0, GL_POSITION, light_position );
  glEnable ( GL_LIGHT0);
}

