// nUtil - An utility Library for gnurbs
// Copyright (C) 2008-2018 Eric Bechet
//
// See the LICENSE file for contributions and license information.
// Please report all bugs and problems to <bechet@cadxfem.org>.
//


#ifndef __NDISPLAY_BASE_H
#define __NDISPLAY_BASE_H

#include "npoint.h"
#include "ndata.h"



enum {DRAW_NORMAL,DRAW_PICK,DRAW_SELECT};

struct pickinfo
{
  int type;
  int id;
  int nb_uid;
  int uids[3];
  unsigned int z;
  bool operator <(const pickinfo & other) const
  {
    return (z+type)<(other.z+other.type);
  }
};

class ndrawable
{
public:
  virtual ~ndrawable() {}
  /// \brief Discretizes the entity.
  /// \param[in] data data to fill-in.
  virtual void Display(data_container& data) const = 0;

  virtual ndrawable* clone() const =0 ;

  /// \brief Returns the dimension of the object (0 = point, 1=line , 2=surface)
  /// \return dimension.
  virtual int dim(void) const =0;
};


class ncallbacks
{
public :
  virtual ~ncallbacks() {}
  // called when elements are picked. Returns which one is of interest (0 by default = first one on stack)
  // return -1 if pick is not desired
  virtual int pick_callback(int, pickinfo[]) {return 0;}
  // called when pick succeeded and draw. Return 0 if no redraw is needed, 1 else.
  virtual int drag_callback(npoint3,npoint3,pickinfo) {return 0;}
  // called upon release
  virtual int release_callback(npoint3,npoint3,pickinfo) {return 0;}
  virtual int key_callback(int key) { return 0;}
  virtual int key_callback(const char *key) { return 0;}
  virtual void draw() {}
  virtual const std::vector<data_container>* get_data() {return NULL;}
protected :
  ncallbacks() {};// never instantiate this type
  npoint3 pickedpoint;
};

class ndisplay_base
{
public:
  ndisplay_base(color c=color(25, 51, 102,255),char *nom=NULL): allow_pick(false),ncb(NULL) {} // constructor
  virtual ~ndisplay_base(){} // destructor
  virtual void init_data(const data_container &data) =0;   // all
  virtual void init_data(const std::vector<data_container> &data)=0;   // all
  virtual void display(bool perspectiveon=0)=0;   // display loop
  virtual void save(const char *fname="default.d3d"){};
  virtual int getnumpick(void) {return 0;} // return max number of pickable elements
  virtual void setpick(bool p) {allow_pick=p;}
  virtual void setcallbacks(ncallbacks *cb) {ncb=cb;}
  bool allow_pick;
  ncallbacks *ncb;
};


#endif // __NDISPLAY_BASE_H
